@@ -30,7 +30,7 @@ from .zhmccli import cli
 from ._helper import print_properties, print_resources, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     click_exception, add_options, LIST_OPTIONS, TABLE_FORMATS, hide_property, \
-    required_option, abort_if_false, validate
+    required_option, abort_if_false, validate, print_dicts
 
 
 POWER_SAVING_TYPES = ['high-performance', 'low-power', 'custom']
@@ -45,34 +45,34 @@ DEFAULT_DPM_FORMAT = 'yaml'
 # JSON schema for adapter mapping file
 MAPPING_SCHEMA = {
     "title": "adapter mapping file schema",
-    "description": "JSON schema that defines the structure of an adapter "
-                   "mapping file.",
+    "description":
+        "JSON schema that defines the structure of an adapter mapping file.",
     "type": "object",
     "additionalProperties": False,
     "required": [
-        "adapter_mapping_file"
+        "adapter-mapping"
     ],
     "properties": {
-        "adapter_mapping_file": {
-            "description": "List of PCHID mappings from config file to import "
-                           "into CPC",
+        "adapter-mapping": {
+            "description":
+                "List of PCHID mappings from config file to import into CPC.",
             "type": "array",
             "items": {
                 "type": "object",
                 "additionalProperties": False,
                 "required": [
-                    "config",
-                    "import",
+                    "old-adapter-id",
+                    "new-adapter-id",
                 ],
                 "properties": {
-                    "config": {
-                        "description": "PCHID of adapter in DPM "
-                                       "configuration file",
+                    "old-adapter-id": {
+                        "description":
+                            "PCHID of adapter in DPM configuration file.",
                         "type": "string"
                     },
-                    "import": {
-                        "description": "PCHID of adapter when imported into "
-                                       "the CPC",
+                    "new-adapter-id": {
+                        "description":
+                            "PCHID of adapter when imported into the CPC.",
                         "type": "string"
                     }
                 }
@@ -82,7 +82,8 @@ MAPPING_SCHEMA = {
 }
 
 
-def help_dpm_file(cmd_ctx, value):
+def help_dpm_file(cmd_ctx, param, value):
+    # pylint: disable=unused-argument
     """
     Click callback function for --help-dpm-file option, that displays
     help for the format of the DPM configuration file and exits.
@@ -118,7 +119,8 @@ corresponding options when issuing the 'Import DPM Configuration' operation.
     cmd_ctx.exit()
 
 
-def help_mapping_file(cmd_ctx, value):
+def help_mapping_file(cmd_ctx, param, value):
+    # pylint: disable=unused-argument
     """
     Click callback function for --help-dpm-file option, that displays
     help for the format of the adapter mapping file and exits.
@@ -128,9 +130,11 @@ def help_mapping_file(cmd_ctx, value):
     print("""
 Format of adapter mapping file:
 
-The adapter mapping file specifies how the PCHIDs of adapters in a DPM
+The adapter mapping file specifies how PCHIDs of adapters in a DPM
 configuration file need to be replaced when importing the DPM configuration
-into a CPC.
+into a new CPC. If you do not provide a mapping for an adapter, DPM uses a
+one-to-one mapping of adapters in the configuration file to adapters on the
+target system.
 
 The adapter mapping file is created manually and is read by the
 'zhmc cpc dpm-import' command.
@@ -138,18 +142,16 @@ The adapter mapping file is created manually and is read by the
 The adapter mapping file is in YAML format and has the following structure:
 
     adapter-mapping:
+      - old-adapter-id:  A11  # PCHID in the DPM configuration file
+        new-adapter-id: "911" # PCHID in the new system that maps to A11
+      - old-adapter-id: "11c"
+        new-adapter-id: "12C"
+      - ... # More mappings, one for each adapter PCHID you intend to map
 
-      - config: A11     # PCHID in the DPM configuration file (used to set the
-                        # 'old-adapter-id' field)
-        import: "911"   # PCHID to be used when importing to a CPC (used to set
-                        # the 'new-adapter-id' field)
-
-      - ...             # More mappings, one for each adapter PCHID
-
-The values of the 'config' and 'import' properties are the PCHID values as
-a hexadecimal string. Therefore, hexadecimal values that consist only of
-decimal digits (like '911' in the example above) must be put into double quotes.
-The characters in the hexadecimal string may be in upper or lower case.
+The values of the 'old-adapter-id' and 'new-adapter-id' properties are the
+PCHID values as a hexadecimal string. Therefore, hexadecimal values that consist
+only of decimal digits (like '911' in the example above) must be put into double
+quotes. The characters in the hexadecimal string may be in upper or lower case.
 """)
     cmd_ctx.exit()
 
@@ -226,8 +228,8 @@ def cpc_show(cmd_ctx, cpc, **options):
               help='The new description of the CPC. '
               '(DPM mode only).')
 @click.option('--acceptable-status', type=str, required=False,
-              help='The new set of acceptable operational status values.')
-# TODO: Support multiple values for acceptable-status
+              help='The new set of acceptable operational status values, as a '
+              'comma-separated list. The empty string specifies an empty list.')
 @click.option('--next-activation-profile', type=str, required=False,
               help='The name of the new next reset activation profile. '
               '(not in DPM mode).')
@@ -253,10 +255,6 @@ def cpc_update(cmd_ctx, cpc, **options):
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
     'zhmc' command name.
-
-    \b
-    Limitations:
-      * The --acceptable-status option does not support multiple values.
     """
     cmd_ctx.execute_cmd(lambda: cmd_cpc_update(cmd_ctx, cpc, options))
 
@@ -579,6 +577,7 @@ def cmd_cpc_update(cmd_ctx, cpc_name, options):
         'processor-time-slice': None,
         'wait-ends-slice': None,
         'no-wait-ends-slice': None,
+        'acceptable-status': None,
     }
     org_options = original_options(options)
     properties = options_to_properties(org_options, name_map)
@@ -600,6 +599,11 @@ def cmd_cpc_update(cmd_ctx, cpc_name, options):
         properties['does-wait-state-end-time-slice'] = \
             org_options['wait-ends-slice']
 
+    if org_options['acceptable-status'] is not None:
+        status_list = org_options['acceptable-status'].split(',')
+        status_list = [item for item in status_list if item]
+        properties['acceptable-status'] = status_list
+
     if not properties:
         cmd_ctx.spinner.stop()
         click.echo("No properties specified for updating CPC '{c}'.".
@@ -746,12 +750,22 @@ def cmd_dpm_import(cmd_ctx, cpc_name, options):
         if 'adapter-mapping' in config_dict:
             del config_dict['adapter-mapping']
 
-    cpc.import_dpm_configuration(config_dict)
+    result = cpc.import_dpm_configuration(config_dict)
 
     cmd_ctx.spinner.stop()
-    click.echo("Imported DPM configuration from DPM config file {f} into "
-               "CPC '{c}'.".
-               format(c=cpc_name, f=dpm_file))
+
+    if result:
+        click.echo("Partially imported DPM configuration from DPM config file "
+                   "{f} into CPC '{c}'. The following parts were not restored:".
+                   format(c=cpc_name, f=dpm_file))
+        try:
+            print_dicts(cmd_ctx, result['output'], cmd_ctx.output_format)
+        except zhmcclient.Error as exc:
+            raise click_exception(exc, cmd_ctx.error_format)
+    else:
+        click.echo("Imported DPM configuration from DPM config file {f} into "
+                   "CPC '{c}'.".
+                   format(c=cpc_name, f=dpm_file))
 
 
 def convert_adapter_mapping(cmd_ctx, mapping_obj):
@@ -761,15 +775,15 @@ def convert_adapter_mapping(cmd_ctx, mapping_obj):
     Configuration operation.
     """
     try:
-        validate(mapping_obj, MAPPING_SCHEMA, "adapter mappping file")
+        validate(mapping_obj, MAPPING_SCHEMA, "adapter mapping file")
     except ValueError as exc:
         raise click_exception(exc, cmd_ctx.error_format)
     ret_mapping = []
-    mapping_list = mapping_obj['adapter_mapping_file']
+    mapping_list = mapping_obj['adapter-mapping']
     for mapping in mapping_list:
         ret_item = {}
-        ret_item['new-adapter-id'] = mapping['import']
-        ret_item['old-adapter-id'] = mapping['config']
+        ret_item['new-adapter-id'] = mapping['new-adapter-id']
+        ret_item['old-adapter-id'] = mapping['old-adapter-id']
         ret_mapping.append(ret_item)
     return ret_mapping
 
