@@ -26,7 +26,7 @@ from .zhmccli import cli, CONSOLE_LOGGER_NAME
 from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     part_console, click_exception, add_options, LIST_OPTIONS, TABLE_FORMATS, \
-    hide_property, ASYNC_TIMEOUT_OPTIONS
+    hide_property, ASYNC_TIMEOUT_OPTIONS, API_VERSION_HMC_2_14_0
 from ._cmd_cpc import find_cpc
 
 
@@ -39,7 +39,7 @@ def find_lpar(cmd_ctx, client, cpc_or_name, lpar_name):
     else:
         cpc_name = cpc_or_name
 
-    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+    if client.version_info() >= API_VERSION_HMC_2_14_0:
         # This approach is faster than going through the CPC.
         # In addition, starting with HMC API version 3.6 (an update to
         # HMC 2.15.0), this approach supports users that do not have object
@@ -112,9 +112,15 @@ def lpar_show(cmd_ctx, cpc, lpar, **options):
     """
     Show details of an LPAR in a CPC.
 
+    The following properties are shown in addition to those returned by the HMC:
+
     \b
+      - 'parent-name' - Name of the parent CPC.
+
     In table output formats, the following properties are hidden by default
     but can be shown by using the --all option:
+
+    \b
       - program-status-word-information
 
     In addition to the command-specific options shown in this help text, the
@@ -128,8 +134,8 @@ def lpar_show(cmd_ctx, cpc, lpar, **options):
 @click.argument('CPC', type=str, metavar='CPC')
 @click.argument('LPAR', type=str, metavar='LPAR')
 @click.option('--acceptable-status', type=str, required=False,
-              help='The new set of acceptable operational status values.')
-# TODO: Support multiple values for acceptable-status
+              help='The new set of acceptable operational status values, as a '
+              'comma-separated list. The empty string specifies an empty list.')
 @click.option('--next-activation-profile', type=str, required=False,
               help='The name of the new next image or load activation '
               'profile.')
@@ -176,7 +182,6 @@ def lpar_update(cmd_ctx, cpc, lpar, **options):
 
     \b
     Limitations:
-      * The --acceptable-status option does not support multiple values.
       * The processor capping/sharing/weight related properties cannot be
         updated.
       * The network-related properties for zaware and ssc cannot beupdated.
@@ -329,6 +334,66 @@ def lpar_psw_restart(cmd_ctx, cpc, lpar, **options):
                                                      options))
 
 
+@lpar_group.command('reset-clear', options_metavar=COMMAND_OPTIONS_METAVAR)
+@click.argument('CPC', type=str, metavar='CPC')
+@click.argument('LPAR', type=str, metavar='LPAR')
+@click.option('--force', is_flag=True, required=False,
+              help='Controls whether this command is permitted when the '
+              'LPAR is in "operating" status.')
+@click.option('--os-ipl-token', type=str, required=False,
+              help='Applicable only to z/OS, this parameter requests that this '
+              'operation only be performed if the provided value matches '
+              'the current value of the os-ipl-token property of the LPAR.')
+@click.option('--allow-status-exceptions', is_flag=True, required=False,
+              help='Allow status "exceptions" as a valid end status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
+@click.pass_obj
+def lpar_reset_clear(cmd_ctx, cpc, lpar, **options):
+    """
+    Resets an LPAR and clears its memory.
+
+    This includes clearing its pending interruptions, resetting its channel
+    subsystem and resetting its processors, and clearing its memory, using the
+    HMC operation "Reset Clear".
+
+    In addition to the command-specific options shown in this help text, the
+    general options (see 'zhmc --help') can also be specified right after the
+    'zhmc' command name.
+    """
+    cmd_ctx.execute_cmd(lambda: cmd_lpar_reset_clear(cmd_ctx, cpc, lpar,
+                                                     options))
+
+
+@lpar_group.command('reset-normal', options_metavar=COMMAND_OPTIONS_METAVAR)
+@click.argument('CPC', type=str, metavar='CPC')
+@click.argument('LPAR', type=str, metavar='LPAR')
+@click.option('--force', is_flag=True, required=False,
+              help='Controls whether this command is permitted when the '
+              'LPAR is in "operating" status.')
+@click.option('--os-ipl-token', type=str, required=False,
+              help='Applicable only to z/OS, this parameter requests that this '
+              'operation only be performed if the provided value matches '
+              'the current value of the os-ipl-token property of the LPAR.')
+@click.option('--allow-status-exceptions', is_flag=True, required=False,
+              help='Allow status "exceptions" as a valid end status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
+@click.pass_obj
+def lpar_reset_normal(cmd_ctx, cpc, lpar, **options):
+    """
+    Resets an LPAR without clearing its memory.
+
+    This includes clearing its pending interruptions, resetting its channel
+    subsystem and resetting its processors, using the HMC operation
+    "Reset Normal".
+
+    In addition to the command-specific options shown in this help text, the
+    general options (see 'zhmc --help') can also be specified right after the
+    'zhmc' command name.
+    """
+    cmd_ctx.execute_cmd(lambda: cmd_lpar_reset_normal(cmd_ctx, cpc, lpar,
+                                                      options))
+
+
 @lpar_group.command('scsi-load', options_metavar=COMMAND_OPTIONS_METAVAR)
 @click.argument('CPC', type=str, metavar='CPC')
 @click.argument('LPAR', type=str, metavar='LPAR')
@@ -338,7 +403,7 @@ def lpar_psw_restart(cmd_ctx, cpc, lpar, **options):
 @click.option('--load-parameter', type=str, required=False,
               help='Provides additional control over the outcome of a '
               'Load operation. Default: empty')
-@click.option('--disk-partition-id', type=str, required=False,
+@click.option('--disk-partition-id', type=int, required=False,
               help='Provides boot program selector. Default: 0')
 @click.option('--operating-system-specific-load-parameters', type=str,
               required=False, help='Provides specific load parameters. '
@@ -382,7 +447,7 @@ def lpar_scsi_load(cmd_ctx, cpc, lpar, load_address, wwpn, lun, **options):
 @click.option('--load-parameter', type=str, required=False,
               help='Provides additional control over the outcome of a '
               'Load operation. Default: empty')
-@click.option('--disk-partition-id', type=str, required=False,
+@click.option('--disk-partition-id', type=int, required=False,
               help='Provides boot program selector. Default: 0')
 @click.option('--operating-system-specific-load-parameters', type=str,
               required=False, help='Provides specific load parameters. '
@@ -425,7 +490,7 @@ def cmd_lpar_list(cmd_ctx, cpc_name, options):
 
     client = zhmcclient.Client(cmd_ctx.session)
 
-    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+    if client.version_info() >= API_VERSION_HMC_2_14_0:
         # This approach is faster than going through the CPC.
         # In addition, starting with HMC API version 3.6 (an update to
         # HMC 2.15.0), this approach supports users that do not have object
@@ -477,7 +542,8 @@ def cmd_lpar_list(cmd_ctx, cpc_name, options):
         ])
 
     for lpar in lpars:
-        additions['cpc'][lpar.uri] = cpc_name
+        cpc = lpar.manager.parent
+        additions['cpc'][lpar.uri] = cpc.name
 
     try:
         print_resources(cmd_ctx, lpars, cmd_ctx.output_format, show_list,
@@ -499,6 +565,9 @@ def cmd_lpar_show(cmd_ctx, cpc_name, lpar_name, options):
 
     properties = dict(lpar.properties)
 
+    # Add artificial property 'parent-name'
+    properties['parent-name'] = cpc_name
+
     # Hide some long or deeply nested properties in table output formats.
     if not options['all'] and cmd_ctx.output_format in TABLE_FORMATS:
         hide_property(properties, 'program-status-word-information')
@@ -514,6 +583,7 @@ def cmd_lpar_update(cmd_ctx, cpc_name, lpar_name, options):
 
     name_map = {
         'next-activation-profile': 'next-activation-profile-name',
+        'acceptable-status': None,
     }
     org_options = original_options(options)
     properties = options_to_properties(org_options, name_map)
@@ -532,6 +602,11 @@ def cmd_lpar_update(cmd_ctx, cpc_name, lpar_name, options):
     if org_options['ssc-master-password'] == '':
         properties['ssc-master-password'] = None
 
+    if org_options['acceptable-status'] is not None:
+        status_list = org_options['acceptable-status'].split(',')
+        status_list = [item for item in status_list if item]
+        properties['acceptable-status'] = status_list
+
     if not properties:
         cmd_ctx.spinner.stop()
         click.echo("No properties specified for updating LPAR '{p}'.".
@@ -642,6 +717,36 @@ def cmd_lpar_psw_restart(cmd_ctx, cpc_name, lpar_name, options):
     click.echo("PSW restart of LPAR '{p}' is complete.".format(p=lpar_name))
 
 
+def cmd_lpar_reset_clear(cmd_ctx, cpc_name, lpar_name, options):
+    # pylint: disable=missing-function-docstring
+
+    client = zhmcclient.Client(cmd_ctx.session)
+    lpar = find_lpar(cmd_ctx, client, cpc_name, lpar_name)
+
+    try:
+        lpar.reset_clear(wait_for_completion=True, **options)
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
+
+    cmd_ctx.spinner.stop()
+    click.echo("Reset clear of LPAR '{p}' is complete.".format(p=lpar_name))
+
+
+def cmd_lpar_reset_normal(cmd_ctx, cpc_name, lpar_name, options):
+    # pylint: disable=missing-function-docstring
+
+    client = zhmcclient.Client(cmd_ctx.session)
+    lpar = find_lpar(cmd_ctx, client, cpc_name, lpar_name)
+
+    try:
+        lpar.reset_normal(wait_for_completion=True, **options)
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
+
+    cmd_ctx.spinner.stop()
+    click.echo("Reset normal of LPAR '{p}' is complete.".format(p=lpar_name))
+
+
 def cmd_lpar_scsi_load(cmd_ctx, cpc_name, lpar_name, load_address,
                        wwpn, lun, options):
     # pylint: disable=missing-function-docstring
