@@ -26,10 +26,12 @@ from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     click_exception, add_options, LIST_OPTIONS
 from ._cmd_partition import find_partition
+from ._cmd_cpc import find_cpc
 
 
 # Defaults for NIC creation
 SSC_IP_ADDRESS_TYPES = ['ipv4', 'ipv6', 'linklocal', 'dhcp']
+VLAN_TYPES = ['enforced', 'none']
 
 
 def find_nic(cmd_ctx, client, cpc_or_name, partition_name, nic_name):
@@ -83,6 +85,16 @@ def nic_show(cmd_ctx, cpc, partition, nic):
     """
     Show the details of a NIC.
 
+    The following properties are shown in addition to those returned by the HMC:
+
+    \b
+      - 'parent-name' - Name of the parent Partition.
+      - 'virtual-switch-name' - Name of the Virtual Switch for the backing
+        OSA/Hipersocket Adapter
+      - 'network-adapter-name' - Name of the backing Adapter
+      - 'network-adapter-port-name' - Name of the backing Adapter Port
+      - 'network-adapter-port-index' - Index of the backing Adapter Port
+
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
     'zhmc' command name.
@@ -113,26 +125,33 @@ def nic_show(cmd_ctx, cpc, partition, nic):
 @click.option('--ssc-management-nic', type=bool, required=False,
               help='Indicates that this NIC should be used as a management '
               'NIC for Secure Service Container to access the web interface. '
-              'Only applicable to NICs of ssc type partitions. '
+              'Only applicable to NICs of SSC partitions. '
               'Default: False')
 @click.option('--ssc-ip-address-type', type=click.Choice(SSC_IP_ADDRESS_TYPES),
               required=False,
               help='Secure Service Container IP address type. '
               'Only applicable to and required for NICs of '
-              'ssc type partitions.')
+              'SSC partitions.')
 @click.option('--ssc-ip-address', type=str, required=False,
               help='IP Address of the SSC management web interface. '
-              'Only applicable to and required for NICs of ssc type '
+              'Only applicable to and required for NICs of SSC '
               'partitions when ssc-ip-address-type is ipv4 or ipv6.')
 @click.option('--ssc-mask-prefix', type=str, required=False,
               help='Network Mask of the SSC management NIC. '
-              'Only applicable to and required for NICs of ssc type '
+              'Only applicable to and required for NICs of SSC '
               'partitions when ssc-ip-address-type is ipv4 or ipv6.')
-@click.option('--vlan-id', type=str, required=False,
-              help='VLAN ID of the SSC management NIC. '
-              'Empty string sets no VLAN ID. '
-              'Only applicable to NICs of ssc type partitions. '
+@click.option('--vlan-id', metavar='[INT|none]', type=str, required=False,
+              help='VLAN ID of the NIC, or "none" for setting no VLAN ID. '
+              'On z14 or later CPCs, specifying a VLAN ID requires using the '
+              '--vlan-type option. '
+              'Only applicable to management NICs of SSC partitions, and to '
+              'OSA and Hipersocket NICs of non-SSC partitions. '
               'Default: No VLAN ID')
+@click.option('--vlan-type', required=False, type=click.Choice(VLAN_TYPES),
+              help='VLAN type of the NIC, or "none" for setting no VLAN type. '
+              'Only supported on z14 or later CPCs, and required if specifying '
+              'a VLAN ID with the --vlan-id option. '
+              'Default: No VLAN type')
 @click.pass_obj
 def nic_create(cmd_ctx, cpc, partition, **options):
     """
@@ -174,21 +193,27 @@ def nic_create(cmd_ctx, cpc, partition, **options):
 @click.option('--ssc-management-nic', type=bool, required=False,
               help='Indicates that this NIC should be used as a management '
               'NIC for Secure Service Container to access the web interface. '
-              'Only applicable to NICs of ssc type partitions. ')
+              'Only applicable to NICs of SSC partitions. ')
 @click.option('--ssc-ip-address-type', type=click.Choice(SSC_IP_ADDRESS_TYPES),
               required=False,
               help='Secure Service Container IP address type. '
-              'Only applicable to NICs of ssc type partitions.')
+              'Only applicable to NICs of SSC partitions.')
 @click.option('--ssc-ip-address', type=str, required=False,
               help='IP Address of the SSC management web interface. '
-              'Only applicable to NICs of ssc type partitions.')
+              'Only applicable to NICs of SSC partitions.')
 @click.option('--ssc-mask-prefix', type=str, required=False,
               help='Network Mask of the SSC management NIC. '
-              'Only applicable to NICs of ssc type partitions.')
-@click.option('--vlan-id', type=str, required=False,
-              help='VLAN ID of the SSC management NIC. '
-              'Empty string sets no VLAN ID. '
-              'Only applicable to NICs of ssc type partitions.')
+              'Only applicable to NICs of SSC partitions.')
+@click.option('--vlan-id', metavar='[INT|none]', type=str, required=False,
+              help='VLAN ID of the NIC, or "none" for setting no VLAN ID. '
+              'On z14 or later CPCs, specifying a VLAN ID requires using the '
+              '--vlan-type option. '
+              'Only applicable to management NICs of SSC partitions, and to '
+              'OSA and Hipersocket NICs of non-SSC partitions.')
+@click.option('--vlan-type', required=False, type=click.Choice(VLAN_TYPES),
+              help='VLAN type of the NIC, or "none" for setting no VLAN type. '
+              'Only supported on z14 or later CPCs, and required if specifying '
+              'a VLAN ID with the --vlan-id option.')
 @click.pass_obj
 def nic_update(cmd_ctx, cpc, partition, nic, **options):
     """
@@ -397,7 +422,45 @@ def cmd_nic_show(cmd_ctx, cpc_name, partition_name, nic_name):
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
-    print_properties(cmd_ctx, nic.properties, cmd_ctx.output_format)
+    properties = dict(nic.properties)
+
+    # Add artificial property 'parent-name'
+    properties['parent-name'] = partition_name
+
+    # Add artificial properties in case of a vswitch-based NIC (OSA, HS)
+    try:
+        vswitch_uri = nic.get_property('virtual-switch-uri')
+    except KeyError:
+        pass
+    else:
+        vswitch_props = client.session.get(vswitch_uri)
+        properties['virtual-switch-name'] = vswitch_props['name']
+
+        cpc = find_cpc(cmd_ctx, client, cpc_name)
+        adapter_uri = vswitch_props['backing-adapter-uri']
+        adapter = cpc.adapters.resource_object(adapter_uri)
+        properties['network-adapter-name'] = adapter.name
+
+        port_index = vswitch_props['port']
+        properties['network-adapter-port-index'] = port_index
+
+        port = adapter.ports.find(index=port_index)
+        properties['network-adapter-port-name'] = port.name
+
+    # Add artificial properties in case of an adapter-based NIC (RoCE, CNA)
+    try:
+        port_uri = nic.get_property('network-adapter-port-uri')
+    except KeyError:
+        pass
+    else:
+        port_props = client.session.get(port_uri)
+        properties['network-adapter-port-name'] = port_props['name']
+        properties['network-adapter-port-index'] = port_props['index']
+
+        adapter_props = client.session.get(port_props['parent'])
+        properties['network-adapter-name'] = adapter_props['name']
+
+    print_properties(cmd_ctx, properties, cmd_ctx.output_format)
 
 
 def cmd_nic_create(cmd_ctx, cpc_name, partition_name, options):
@@ -415,7 +478,7 @@ def cmd_nic_create(cmd_ctx, cpc_name, partition_name, options):
     org_options = original_options(options)
     properties = options_to_properties(org_options, name_map)
 
-    set_vlan_id(cmd_ctx, properties, org_options)
+    set_vlan_id_type(cmd_ctx, properties, org_options)
 
     properties.update(backing_uri(
         cmd_ctx, partition.manager.cpc, org_options, required=True))
@@ -445,7 +508,7 @@ def cmd_nic_update(cmd_ctx, cpc_name, partition_name, nic_name, options):
     org_options = original_options(options)
     properties = options_to_properties(org_options, name_map)
 
-    set_vlan_id(cmd_ctx, properties, org_options)
+    set_vlan_id_type(cmd_ctx, properties, org_options)
 
     uri_prop = backing_uri(
         cmd_ctx, nic.manager.partition.manager.cpc, org_options)
@@ -486,12 +549,15 @@ def cmd_nic_delete(cmd_ctx, cpc_name, partition_name, nic_name):
     click.echo("NIC '{n}' has been deleted.".format(n=nic_name))
 
 
-def set_vlan_id(cmd_ctx, properties, org_options):
+def set_vlan_id_type(cmd_ctx, properties, org_options):
     """
-    Set the 'vlan-id' property from the options.
+    Set the 'vlan-id' and 'vlan-type' properties from the options.
     """
     vlan_id = org_options['vlan-id']
-    if vlan_id == '':
+    if vlan_id is None:
+        pass  # No change on update, use HMC default on create
+    elif vlan_id == 'none':
+        # Reset to no VLAN ID (important for update)
         properties['vlan-id'] = None
     else:
         try:
@@ -500,3 +566,12 @@ def set_vlan_id(cmd_ctx, properties, org_options):
             raise click_exception(
                 "Invalid value for '--vlan-id': {} is not a valid integer".
                 format(vlan_id), cmd_ctx.error_format)
+
+    vlan_type = org_options['vlan-type']
+    if vlan_type is None:
+        pass  # No change on update, use HMC default on create
+    elif vlan_type == 'none':
+        # Reset to no VLAN type (important for update)
+        properties['vlan-type'] = None
+    else:
+        properties['vlan-type'] = vlan_type
