@@ -31,7 +31,7 @@ from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     part_console, click_exception, storage_management_feature, \
     add_options, LIST_OPTIONS, TABLE_FORMATS, hide_property, \
-    ASYNC_TIMEOUT_OPTIONS
+    ASYNC_TIMEOUT_OPTIONS, API_VERSION_HMC_2_14_0
 from ._cmd_cpc import find_cpc
 from ._cmd_storagegroup import find_storagegroup
 from ._cmd_metrics import get_metric_values
@@ -59,7 +59,7 @@ def find_partition(cmd_ctx, client, cpc_or_name, partition_name):
     else:
         cpc_name = cpc_or_name
 
-    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+    if client.version_info() >= API_VERSION_HMC_2_14_0:
         # This approach is faster than going through the CPC.
         # In addition, this approach supports users that do not have object
         # access permission to the parent CPC of the LPAR.
@@ -139,9 +139,17 @@ def partition_show(cmd_ctx, cpc, partition, **options):
     """
     Show the details of a partition in a CPC.
 
+    The following properties are shown in addition to those returned by the HMC:
+
     \b
+      - 'parent-name' - Name of the parent CPC.
+      - 'nic-names' - Names of the NICs referenced by 'nic-uris'
+        (index-correlated).
+
     In table output formats, the following properties are hidden by default
     but can be shown by using the --all option:
+
+    \b
       - crypto-configuration
 
     In addition to the command-specific options shown in this help text, the
@@ -245,6 +253,9 @@ def partition_dump(cmd_ctx, cpc, partition, **options):
               help='The name of the new partition.')
 @click.option('--description', type=str, required=False,
               help='The description of the new partition.')
+@click.option('--acceptable-status', type=str, required=False,
+              help='The set of acceptable operational status values, as a '
+              'comma-separated list. The empty string specifies an empty list.')
 @click.option('--cp-processors', type=int, required=False,
               help='The number of general purpose (CP) processors. '
               'Default: No CP processors')
@@ -388,6 +399,9 @@ def partition_create(cmd_ctx, cpc, **options):
               help='The new name of the partition.')
 @click.option('--description', type=str, required=False,
               help='The new description of the partition.')
+@click.option('--acceptable-status', type=str, required=False,
+              help='The new set of acceptable operational status values, as a '
+              'comma-separated list. The empty string specifies an empty list.')
 @click.option('--cp-processors', type=int, required=False,
               help='The new number of general purpose (CP) processors.')
 @click.option('--ifl-processors', type=int, required=False,
@@ -442,8 +456,9 @@ def partition_create(cmd_ctx, cpc, **options):
 @click.option('--boot-media-file', type=str, required=False,
               help='Boot from removable media on the HMC: The path to the '
               'image file on the HMC.')
-@click.option('--boot-iso', type=str, required=False,
-              help='Boot from an ISO image mounted to this partition.')
+@click.option('--boot-iso', is_flag=True, required=False,
+              help='Boot from an ISO image mounted to this partition. '
+              'The ISO image can be mounted using "zhmc partition mountiso".')
 @click.option('--secure-boot', type=bool, required=False,
               help='Check the software signature of what is booted against '
               'what the distributor signed it with. '
@@ -734,7 +749,7 @@ Help for usage related options of the partition list command:
 
     client = zhmcclient.Client(cmd_ctx.session)
 
-    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+    if client.version_info() >= API_VERSION_HMC_2_14_0:
         # This approach is faster than going through the CPC.
         # In addition, this approach supports users that do not have object
         # access permission to the parent CPC of the LPAR.
@@ -925,10 +940,20 @@ def cmd_partition_show(cmd_ctx, cpc_name, partition_name, options):
 
     properties = dict(partition.properties)
 
+    # Add artificial property 'parent-name'
+    properties['parent-name'] = cpc_name
+
     # Hide some long or deeply nested properties in table output formats.
     if not options['all'] and cmd_ctx.output_format in TABLE_FORMATS:
         hide_property(properties, 'crypto-configuration')
 
+    # Add artificial property 'nic-names'
+    nic_names = []
+    for nic_uri in partition.properties['nic-uris']:
+        nic_props = client.session.get(nic_uri)
+        nic_names.append(nic_props['name'])
+    properties['nic-names'] = nic_names
+
     print_properties(cmd_ctx, properties, cmd_ctx.output_format)
 
 
@@ -988,8 +1013,15 @@ def cmd_partition_create(cmd_ctx, cpc_name, options):
         'boot-media-file',
     )
 
+    # The acceptable-status option.
+    # For consistency, a list is used even though it is a single option.
+    status_option_names = (
+        'acceptable-status',
+    )
+
     # Options handled in this function
-    special_opt_names = boot_ftp_option_names + boot_media_option_names
+    special_opt_names = boot_ftp_option_names + boot_media_option_names + \
+        status_option_names
     name_map = dict((opt, None) for opt in special_opt_names)
 
     org_options = original_options(options)
@@ -1050,6 +1082,11 @@ def cmd_partition_create(cmd_ctx, cpc_name, options):
     if org_options['ssc-ipv4-gateway'] == '':
         properties['ssc-ipv4-gateway'] = None
 
+    if org_options['acceptable-status'] is not None:
+        status_list = org_options['acceptable-status'].split(',')
+        status_list = [item for item in status_list if item]
+        properties['acceptable-status'] = status_list
+
     try:
         new_partition = cpc.partitions.create(properties)
     except zhmcclient.Error as exc:
@@ -1110,11 +1147,18 @@ def cmd_partition_update(cmd_ctx, cpc_name, partition_name, options):
         'boot-iso',
     )
 
+    # The acceptable-status option.
+    # For consistency, a list is used even though it is a single option.
+    status_option_names = (
+        'acceptable-status',
+    )
+
     # Options handled in this function
     special_opt_names = \
         boot_storage_option_names + old_boot_storage_option_names + \
         boot_network_option_names + boot_ftp_option_names + \
-        boot_media_option_names + boot_iso_option_names
+        boot_media_option_names + boot_iso_option_names + \
+        status_option_names
     name_map = dict((opt, None) for opt in special_opt_names)
 
     org_options = original_options(options)
@@ -1149,7 +1193,7 @@ def cmd_partition_update(cmd_ctx, cpc_name, partition_name, options):
 
     used_boot_iso_opts = [
         '--' + name for name in boot_iso_option_names
-        if org_options[name] is not None]
+        if org_options[name]]  # is_flag options default to False
 
     if used_boot_storage_opts and used_old_boot_storage_opts:
         raise click_exception(
@@ -1243,7 +1287,11 @@ def cmd_partition_update(cmd_ctx, cpc_name, partition_name, options):
 
     elif used_boot_iso_opts:
         properties['boot-device'] = 'iso-image'
-
+        # Note: 'boot-iso-image-name' is read-only and is changed by using
+        #       the Mount/Unmount ISO operations.
+        # Note: 'boot-iso-ins-file' is writeable, for using a different
+        #       INS file than the one specified in Mount ISO. This is not
+        #       currently supported by zhmccli.
     else:
         # boot-device="none" is the default
         pass
@@ -1260,6 +1308,11 @@ def cmd_partition_update(cmd_ctx, cpc_name, partition_name, options):
     if org_options['secure-boot']:
         properties['secure-boot'] = True
 
+    if org_options['acceptable-status'] is not None:
+        status_list = org_options['acceptable-status'].split(',')
+        status_list = [item for item in status_list if item]
+        properties['acceptable-status'] = status_list
+
     if not properties:
         cmd_ctx.spinner.stop()
         click.echo("No properties specified for updating partition '{p}'.".
