@@ -21,6 +21,8 @@ from __future__ import absolute_import
 import json
 from collections import OrderedDict
 import sys
+import os
+import shutil
 import threading
 import re
 import jsonschema
@@ -32,6 +34,18 @@ from tabulate import tabulate
 import zhmcclient
 import zhmcclient_mock
 
+# HMC API versions for new HMC versions
+# Can be used for comparison with Client.version_info()
+API_VERSION_HMC_2_11_1 = (1, 1)
+API_VERSION_HMC_2_12_0 = (1, 3)
+API_VERSION_HMC_2_12_1 = (1, 4)
+API_VERSION_HMC_2_13_0 = (1, 6)
+API_VERSION_HMC_2_13_1 = (1, 7)
+API_VERSION_HMC_2_14_0 = (2, 20)
+API_VERSION_HMC_2_14_1 = (2, 35)
+API_VERSION_HMC_2_15_0 = (3, 1)
+API_VERSION_HMC_2_16_0 = (4, 1)
+
 # Display of options in usage line
 GENERAL_OPTIONS_METAVAR = '[GENERAL-OPTIONS]'
 COMMAND_OPTIONS_METAVAR = '[COMMAND-OPTIONS]'
@@ -118,6 +132,48 @@ ASYNC_TIMEOUT_OPTIONS = [
                  format(def_ot=zhmcclient.DEFAULT_OPERATION_TIMEOUT)),
 ]
 
+# Env var for overriding the terminal width.
+TERMWIDTH_ENVVAR = 'ZHMCCLI_TERMWIDTH'
+
+# Boundaries for terminal width to be used for Click help messages.
+MIN_TERMINAL_WIDTH = 80
+MAX_TERMINAL_WIDTH = 160
+
+
+def get_click_terminal_width():
+    """
+    Return the terminal width to be used for Click help messages, as an integer.
+    """
+    width = get_terminal_width()
+    width = min(width, MAX_TERMINAL_WIDTH)
+    width = max(width, MIN_TERMINAL_WIDTH)
+    return width
+
+
+def get_terminal_width():
+    """
+    Return the terminal width, as an integer.
+    """
+
+    terminal_width = os.getenv(TERMWIDTH_ENVVAR, None)
+    if terminal_width:
+        try:
+            terminal_width = int(terminal_width)
+            return terminal_width
+        except ValueError:
+            pass
+
+    # We first try shutil.get_terminal_size() which was added in Python 3.3.
+    # Click 8.0 has deprecated click.get_terminal_size() and issues a
+    # DeprecationWarning, but on Python 2.7, Click is pinned to <8.0, so we
+    # can use click.get_terminal_size() without triggering the
+    # DeprecationWarning.
+    try:
+        ts = shutil.get_terminal_size()
+    except AttributeError:
+        ts = click.get_terminal_size()  # pylint: disable=no-member
+    return ts[0]
+
 
 def abort_if_false(ctx, param, value):
     """
@@ -623,13 +679,13 @@ def print_resources_as_table(
                 resource_props[name] = value
                 prop_names[name] = None
         else:
-            for name in sorted(resource.properties.keys()):
+            for name in sorted(resource.properties):
                 # May raise zhmcclient exceptions
                 resource_props[name] = resource.prop(name)
                 prop_names[name] = None
         if all:
             resource.pull_full_properties()
-            for name in resource.properties.keys():
+            for name in resource.properties:
                 if name not in prop_names:
                     # May raise zhmcclient exceptions
                     resource_props[name] = resource.prop(name)
@@ -713,12 +769,12 @@ def print_dicts_as_table(
                 dict_props[name] = value
                 prop_names[name] = None
         else:
-            for name in sorted(_dict.keys()):
+            for name in sorted(_dict):
                 # May raise zhmcclient exceptions
                 dict_props[name] = _dict[name]
                 prop_names[name] = None
         if all:
-            for name in _dict.keys():
+            for name in _dict:
                 if name not in prop_names:
                     # May raise zhmcclient exceptions
                     dict_props[name] = _dict[name]
@@ -842,7 +898,7 @@ def print_properties_as_json(cmd_ctx, properties, show_list=None):
         If `None`, all properties in the `properties` dict are shown.
     """
     show_properties = OrderedDict()
-    for pname in properties:
+    for pname in sorted(properties):
         if show_list is None or pname in show_list:
             show_properties[pname] = properties[pname]
     json_str = json.dumps(show_properties)
@@ -901,13 +957,13 @@ def print_resources_as_json(
                 resource_props[name] = value
                 prop_names[name] = None
         else:
-            for name in sorted(resource.properties.keys()):
+            for name in resource.properties:
                 # May raise zhmcclient exceptions
                 resource_props[name] = resource.prop(name)
                 prop_names[name] = None
         if all:
             resource.pull_full_properties()
-            for name in resource.properties.keys():
+            for name in resource.properties:
                 if name not in prop_names:
                     # May raise zhmcclient exceptions
                     resource_props[name] = resource.prop(name)
@@ -917,7 +973,7 @@ def print_resources_as_json(
     json_obj = []
     for resource_props in resource_props_list:
         json_res = OrderedDict()
-        for name in prop_names:
+        for name in sorted(prop_names):
             value = resource_props.get(name, None)
             json_res[name] = value
         json_obj.append(json_res)
@@ -970,12 +1026,12 @@ def print_dicts_as_json(
                 dict_props[name] = value
                 prop_names[name] = None
         else:
-            for name in sorted(_dict.keys()):
+            for name in _dict:
                 # May raise zhmcclient exceptions
                 dict_props[name] = _dict[name]
                 prop_names[name] = None
         if all:
-            for name in _dict.keys():
+            for name in _dict:
                 if name not in prop_names:
                     # May raise zhmcclient exceptions
                     dict_props[name] = _dict[name]
@@ -985,7 +1041,7 @@ def print_dicts_as_json(
     json_obj = []
     for dict_props in dict_props_list:
         json_res = OrderedDict()
-        for name in prop_names:
+        for name in sorted(prop_names):
             value = dict_props.get(name, None)
             json_res[name] = value
         json_obj.append(json_res)
